import time
import numpy as np

from gpaw.poisson import PoissonSolver
from gpaw.poisson_extended import ExtendedPoissonSolver
from gpaw.poisson_extravacuum import ExtraVacuumPoissonSolver
from gpaw.grid_descriptor import GridDescriptor


do_output = False
do_plot = False
poissoneps = 1e-16

if do_output:
    def equal(x, y, tol=0):
        res = {True: 'ok', False: 'not ok'}[abs(x - y) < tol]
        print('%.10e vs %.10e at %.10e is %s' % (x, y, tol, res))
else:
    from gpaw.test import equal


# Model grid
N = 16
N_c = np.array((1, 1, 3)) * N
cell_c = N_c / float(N)
gd = GridDescriptor(N_c, cell_c, False)

# Construct model density
coord_vg = gd.get_grid_point_coordinates()
x_g = coord_vg[0, :]
y_g = coord_vg[1, :]
z_g = coord_vg[2, :]
rho_g = gd.zeros()
for z0 in [1, 2]:
    rho_g += 10 * (z_g - z0) * \
        np.exp(-20 * np.sum((coord_vg.T - np.array([.5, .5, z0])).T**2,
                            axis=0))

if do_plot:
    big_rho_g = gd.collect(rho_g)
    if gd.comm.rank == 0:
        import matplotlib.pyplot as plt
        fig, ax_ij = plt.subplots(3, 4, figsize=(20, 10))
        ax_i = ax_ij.ravel()
        ploti = 0
        Ng_c = gd.get_size_of_global_array()
        plt.sca(ax_i[ploti])
        ploti += 1
        plt.pcolormesh(big_rho_g[Ng_c[0] / 2])
        plt.sca(ax_i[ploti])
        ploti += 1
        plt.plot(big_rho_g[Ng_c[0] / 2, Ng_c[1] / 2])


def plot_phi(phi_g):
    if do_plot:
        big_phi_g = gd.collect(phi_g)
        if gd.comm.rank == 0:
            global ploti
            if ploti == 4:
                ploti -= 2
            plt.sca(ax_i[ploti])
            ploti += 1
            plt.pcolormesh(big_phi_g[Ng_c[0] / 2])
            plt.sca(ax_i[ploti])
            ploti += 1
            plt.plot(big_phi_g[Ng_c[0] / 2, Ng_c[1] / 2])
            plt.ylim(np.array([-1, 1]) * 0.15)


def poisson_solve(gd, rho_g, poisson):
    phi_g = gd.zeros()
    rho_g = rho_g
    t0 = time.time()
    npoisson = poisson.solve(phi_g, rho_g)
    t1 = time.time()
    if do_output:
        if gd.comm.rank == 0:
            print('Iterations: %s, Time: %.3f s' % (str(npoisson), t1 - t0))
    return phi_g, npoisson


def poisson_init_solve(gd, rho_g, poisson):
    poisson.set_grid_descriptor(gd)
    phi_g, npoisson = poisson_solve(gd, rho_g, poisson)
    plot_phi(phi_g)
    return phi_g, npoisson


def compare(phi1_g, phi2_g, val, eps=np.sqrt(poissoneps)):
    big_phi1_g = gd.collect(phi1_g)
    big_phi2_g = gd.collect(phi2_g)
    if gd.comm.rank == 0:
        diff = np.max(np.absolute(big_phi1_g - big_phi2_g))
    else:
        diff = 0.0
    diff = np.array([diff])
    gd.comm.broadcast(diff, 0)
    equal(diff[0], val, eps)


# Get reference from default poissonsolver
poisson = PoissonSolver('fd', eps=poissoneps)
phiref_g, npoisson = poisson_init_solve(gd, rho_g, poisson)

# Test agreement with default
poisson = ExtraVacuumPoissonSolver(N_c, PoissonSolver('fd', eps=poissoneps))
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
compare(phi_g, phiref_g, 0.0, 1e-24)

# New reference with extra vacuum
gpts = N_c * 4
poisson = ExtraVacuumPoissonSolver(gpts, PoissonSolver('fd', eps=poissoneps))
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
# print poisson.get_description()
compare(phi_g, phiref_g, 2.6485385144e-02)
phiref_g = phi_g

# Compare to ExtendedPoissonSolver
# TODO: remove this test when/if ExtendedPoissonSolver is deprecated
poisson = ExtendedPoissonSolver(eps=poissoneps,
                                extended={'gpts': gpts / 2,
                                          'useprev': True})
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
compare(phi_g, phiref_g, 0.0, 1e-24)

# Test with single coarsening
poisson = ExtraVacuumPoissonSolver(gpts, PoissonSolver('fd', eps=poissoneps),
                                   PoissonSolver('fd', eps=poissoneps), 1)
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
compare(phi_g, phiref_g, 1.5043946611e-04)

# Test with two coarsenings
poisson = ExtraVacuumPoissonSolver(gpts, PoissonSolver('fd', eps=poissoneps),
                                   PoissonSolver('fd', eps=poissoneps), 2)
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
compare(phi_g, phiref_g, 1.2980906205e-03)

# Test with cascaded single coarsenings
poisson1 = ExtraVacuumPoissonSolver(gpts / 2,
                                    PoissonSolver('fd', eps=poissoneps),
                                    PoissonSolver('fd', eps=poissoneps), 1)
poisson = ExtraVacuumPoissonSolver(gpts / 2, poisson1,
                                   PoissonSolver('fd', eps=poissoneps), 1)
phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
# print poisson.get_description()
compare(phi_g, phiref_g, 1.7086531461e-04)

# Test auxgrid
gpts = N_c * 4
for coarses in [1, 2, 3]:
    for nn_refine in [1, 3]:
        for nn_laplace in [1, 3]:
            EVPS = ExtraVacuumPoissonSolver
            poisson = EVPS(gpts, PoissonSolver('fd', eps=poissoneps),
                           PoissonSolver('fd', eps=poissoneps), coarses,
                           use_aux_grid=False,
                           nn_refine=nn_refine, nn_laplace=nn_laplace)
            phiref_g, npoisson = poisson_init_solve(gd, rho_g, poisson)

            poisson = EVPS(gpts, PoissonSolver('fd', eps=poissoneps),
                           PoissonSolver('fd', eps=poissoneps), coarses,
                           use_aux_grid=True,
                           nn_refine=nn_refine, nn_laplace=nn_laplace)
            phi_g, npoisson = poisson_init_solve(gd, rho_g, poisson)
            compare(phi_g, phiref_g, 0.0, 1e-14)

if do_plot:
    if gd.comm.rank == 0:
        plt.show()



from ase import Atoms
from gpaw import GPAW
from gpaw.mpi import world, serial_comm

a = Atoms('H',
          cell=(1, 3, 3),
          pbc=1)

a.calc = GPAW(mode='pw',
              h=0.15,
              kpts=(4, 1, 1),
              basis='dzp',
              nbands=4,
              eigensolver='rmm-diis',
              parallel={'domain': 1})

a.get_potential_energy()
w1 = a.calc.get_pseudo_wave_function(0, 1)
e1 = a.calc.get_eigenvalues(1)

a.calc.write('H')

if world.size <= 2:
    scalapack = None
else:
    mb = world.size // 4
    scalapack = (2, mb, 32)

a.calc.diagonalize_full_hamiltonian(nbands=100, scalapack=scalapack)
w2 = a.calc.get_pseudo_wave_function(0, 1)
e2 = a.calc.get_eigenvalues(1)

calc = GPAW('H', txt=None, parallel={'domain': 1})
calc.diagonalize_full_hamiltonian(nbands=100, scalapack=scalapack)
w3 = calc.get_pseudo_wave_function(0, 1)
e3 = calc.get_eigenvalues(1)

calc.write('Hwf', 'all')

calc = GPAW('Hwf', txt=None, communicator=serial_comm)
w4 = calc.get_pseudo_wave_function(0, 1)
e4 = calc.get_eigenvalues(1)

for w in [w2, w3, w4]:
    err = abs(abs(w[1, 2, 3]) - abs(w1[1, 2, 3]))
    assert err < 1e-7, err

for e in [e2, e3, e4]:
    err = abs(e[0] - e1[0])
    assert err < 2e-9, err
    err = abs(e[-1] - e2[-1])
    assert err < 1e-10, err



import numpy as np

from ase.build import molecule
from gpaw import GPAW
from gpaw.tddft import TDDFT
from gpaw.poisson import PoissonSolver
from gpaw.mpi import world

from gpaw.test import equal

# Atoms
atoms = molecule('SiH4')
atoms.center(vacuum=4.0)

# Ground-state calculation
calc = GPAW(nbands=7, h=0.4,
            poissonsolver=PoissonSolver(eps=1e-16),
            convergence={'density': 1e-8},
            xc='GLLBSC',
            txt='gs.out')
atoms.set_calculator(calc)
energy = atoms.get_potential_energy()
calc.write('gs.gpw', mode='all')

# Time-propagation calculation
td_calc = TDDFT('gs.gpw', txt='td.out')
td_calc.absorption_kick(np.ones(3) * 1e-5)
td_calc.propagate(20, 3, 'dm.dat')

# Write a restart point
td_calc.write('td.gpw', mode='all')

# Keep propagating
td_calc.propagate(20, 3, 'dm.dat')

# Restart from the restart point
td_calc = TDDFT('td.gpw', txt='td2.out')
td_calc.propagate(20, 3, 'dm.dat')
world.barrier()

# Check dipole moment file
data_tj = np.loadtxt('dm.dat')
# Original run
ref_i = data_tj[4:6].ravel()
# Restarted steps
data_i = data_tj[7:].ravel()

tol = 1e-10
equal(data_i, ref_i, tol)

# Test the absolute values
data = np.loadtxt('dm.dat')[:6].ravel()
if 0:
    from gpaw.test import print_reference
    print_reference(data, 'ref', '%.12le')

ref = [0.0, 1.46915686e-15, -3.289312570937e-14, -2.273046460905e-14,
       -3.201827522804e-15, 0.82682747, 1.30253044e-15, 6.113786782415e-05,
       6.113753835177e-05, 6.113827464597e-05, 1.65365493, -1.69353262e-15,
       0.0001073089135539, 0.0001073052457949, 0.0001073068465827, 2.4804824,
       1.42934356e-15, 0.0001353894007493, 0.0001353887214486,
       0.0001353873226291, 3.30730987, -3.43926271e-16, 0.0001441529062519,
       0.000144155244532, 0.0001441536382364, 4.13413733, -8.41062896e-16,
       0.0001348222114923, 0.0001348264801341, 0.0001348229035149]

print(data.tolist())

tol = 1e-7
equal(data, ref, tol)



from __future__ import print_function

import time
import numpy as np
from gpaw.utilities.blas import gemmdot, gemv

def getrand(shape, dtype):
    if isinstance(shape, int):
        shape = (shape,)
    nelements = np.prod(shape)
    if dtype == float:
        return np.random.normal(size=nelements).reshape(shape)
    elif dtype == complex:
        return (np.random.uniform(size=nelements) * np.exp(1j \
            * np.random.uniform(0,2*np.pi,size=nelements))).reshape(shape)
    else:
        raise ValueError('Unsupported dtype "%s".' % dtype)

P = 12
Q = 1456
L = 378
G = 5013

dtype = float
beta = 0.0 # note that beta=0.0 and beta=1.0 have different gemv subroutines
refperf = 1e8 # each test should last roughly 1 s with this performance
test_gemmdot = False # gemmdot is unbelievable slow for matrix-vector products

mem = 0
itemsize = np.nbytes[np.dtype(dtype)]
mem += P*Q*L*itemsize #B_pqL
mem += L*itemsize #Y_L
mem += P*Q*itemsize #BY_pq
mem += Q*G*itemsize #n_qg
mem += G*itemsize #x_g
mem += Q*itemsize #nx_q

print('Estimated memory: %8.5f MB' % (mem/1024**2.,))

B_pqL = getrand((P,Q,L), dtype)
Y_L = getrand(L, dtype)
n_qg = getrand((Q,G), dtype)
x_g = getrand(G, dtype)

# -------------------------------------------------------------------

print('\n%s\nBY_pq calculations\n%s\n' % ('='*40, '='*40))
numflop = (dtype==float and 2 or 8)*P*Q*L
numreps = 1+int(refperf/numflop)

# Reference value
BY0_pq = np.dot(B_pqL, Y_L)

t = time.time()
for n in range(numreps):
    BY1_pq = np.dot(B_pqL, Y_L)
t = time.time()-t
performance = numflop*numreps/t
print('dot    : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(BY0_pq-BY1_pq).max()<5e-12
del BY1_pq

if test_gemmdot:
    BY2_pq = np.empty((P,Q), dtype)
    t = time.time()
    for n in range(numreps):
        BY2_pq.fill(0.0)
        gemmdot(B_pqL, Y_L, 1.0, beta, BY2_pq)
    t = time.time()-t
    performance = numflop*numreps/t
    print('gemmdot: %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
    assert np.abs(BY0_pq-BY2_pq).max()<5e-12
    del BY2_pq

BY3_pq = np.empty((P,Q), dtype)
t = time.time()
for n in range(numreps):
    BY3_pq.fill(0.0)
    gemv(1.0, B_pqL, Y_L, beta, BY3_pq, 't')
t = time.time()-t
performance = numflop*numreps/t
print('gemvT  : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(BY0_pq-BY3_pq).max()<5e-12
del BY3_pq

B_xL = B_pqL.reshape((P*Q,L))
BY4_x = np.empty(P*Q, dtype)
t = time.time()
for n in range(numreps):
    BY4_x.fill(0.0)
    gemv(1.0, B_xL, Y_L, beta, BY4_x, 't')
t = time.time()-t
performance = numflop*numreps/t
print('gemvT2D: %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(BY0_pq-BY4_x.reshape((P,Q))).max()<5e-12
del B_xL, BY4_x

BT_Lqp = B_pqL.T.copy()
BY5T_qp = np.empty((Q,P), dtype)
t = time.time()
for n in range(numreps):
    BY5T_qp.fill(0.0)
    gemv(1.0, BT_Lqp, Y_L, beta, BY5T_qp, 'n')
t = time.time()-t
performance = numflop*numreps/t
print('gemvN  : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(BY0_pq-BY5T_qp.T).max()<5e-12
del BT_Lqp, BY5T_qp

BT_Lx = B_pqL.T.reshape((L,Q*P)).copy()
BY6T_x = np.empty(Q*P, dtype)
t = time.time()
for n in range(numreps):
    BY6T_x.fill(0.0)
    gemv(1.0, BT_Lx, Y_L, beta, BY6T_x, 'n')
t = time.time()-t
performance = numflop*numreps/t
print('gemvN2D: %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(BY0_pq-BY6T_x.reshape((Q,P)).T).max()<5e-12
del BT_Lx, BY6T_x

# -------------------------------------------------------------------

print('\n%s\nnx_q calculations\n%s\n' % ('='*40, '='*40))
numflop = (dtype==float and 2 or 8)*Q*G
numreps = 1+int(refperf/numflop)

# Reference value
nx0_q = np.dot(n_qg, x_g)

t = time.time()
for n in range(numreps):
    nx1_q = np.dot(n_qg, x_g)
t = time.time()-t
performance = numflop*numreps/t
print('dot    : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(nx0_q-nx1_q).max()<5e-12
del nx1_q

if test_gemmdot:
    nx2_q = np.empty(Q, dtype)
    t = time.time()
    for n in range(numreps):
        nx2_q.fill(0.0)
        gemmdot(n_qg, x_g, 1.0, beta, nx2_q)
    t = time.time()-t
    performance = numflop*numreps/t
    print('gemmdot: %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
    assert np.abs(nx0_q-nx2_q).max()<5e-12
    del nx2_q

nx3_q = np.empty(Q, dtype)
t = time.time()
for n in range(numreps):
    nx3_q.fill(0.0)
    gemv(1.0, n_qg, x_g, beta, nx3_q, 't')
t = time.time()-t
performance = numflop*numreps/t
print('gemvT  : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(nx0_q-nx3_q).max()<5e-12
del nx3_q

nT_gq = n_qg.T.copy()
nx4_q = np.empty(Q, dtype)
t = time.time()
for n in range(numreps):
    nx4_q.fill(0.0)
    gemv(1.0, nT_gq, x_g, beta, nx4_q, 'n')
t = time.time()-t
performance = numflop*numreps/t
print('gemvN  : %8.5f s, %8.5f Mflops' % (t,performance/1024**2.))
assert np.abs(nx0_q-nx4_q).max()<5e-12
del nT_gq, nx4_q




import numpy as np

import ase.io
from ase.calculators.vdwcorrection import vdWTkatchenko09prl
from ase.parallel import barrier, parprint
from ase.build import molecule

from gpaw import GPAW
from gpaw.analyse.hirshfeld import HirshfeldPartitioning
from gpaw.analyse.vdwradii import vdWradii
from gpaw.cluster import Cluster
from gpaw.test import equal

h = 0.4
s = Cluster(molecule('LiH'))
s.minimal_box(3., h=h)


def print_charge_and_check(hp, q=0, label='unpolarized'):
    q_a = np.array(hp.get_charges())
    parprint('Charges ({0})='.format(label), q_a, ', sum=', q_a.sum())
    equal(q_a.sum(), q, 0.03)
    return q_a

# spin unpolarized

if 1:
    out_traj = 'LiH.traj'
    out_txt = 'LiH.txt'

    cc = GPAW(h=h, xc='PBE', txt=out_txt)

    # this is needed to initialize txt output
    cc.initialize(s)

    hp = HirshfeldPartitioning(cc)
    c = vdWTkatchenko09prl(hp,
                           vdWradii(s.get_chemical_symbols(), 'PBE'))
    s.set_calculator(c)
    E = s.get_potential_energy()
    F_ac = s.get_forces()
    s.write(out_traj)
    q_a = print_charge_and_check(hp)

    barrier()

    # test I/O, accuracy due to text output
    accuracy = 1.e-5
    for fname in [out_traj, out_txt]:
        s_out = ase.io.read(fname)
        equal(s_out.get_potential_energy(), E, accuracy)
        for fi, fo in zip(F_ac, s_out.get_forces()):
            equal(fi, fo, accuracy)

# spin polarized

if 0:
    ccs = GPAW(h=h, xc='PBE', spinpol=True,
               txt=None)
    hps = HirshfeldPartitioning(ccs)
    cs = vdWTkatchenko09prl(hps, vdWradii(s.get_chemical_symbols(), 'PBE'))
    s.set_calculator(cs)
    Es = s.get_potential_energy()
    Fs_ac = s.get_forces()

    qs_a = print_charge_and_check(hps, label='spin')

    equal(q_a, qs_a, 1.e-6)
    equal(E, Es, 1.e-4)
    equal(F_ac, Fs_ac, 1.e-4)

# charged

if 0:
    cc.set(charge=1)
    hpp = HirshfeldPartitioning(cc)
    cp = vdWTkatchenko09prl(hpp,
                            vdWradii(s.get_chemical_symbols(), 'PBE'))
    s.set_calculator(cp)
    E = s.get_potential_energy()
    
    print_charge_and_check(hpp, 1, label='+1')



from __future__ import print_function

from ase import Atom
from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.test import equal

fname='H2_PBE.gpw'
fwfname='H2_wf_PBE.gpw'
txt = None

# write first if needed
try:
    c = GPAW(fname, txt=txt)
    c = GPAW(fwfname, txt=txt)
except:
    s = Cluster([Atom('H'), Atom('H', [0,0,1])])
    s.minimal_box(3.)
    c = GPAW(xc={'name': 'PBE', 'stencil': 1},
             h=.3, convergence={'density':1e-4, 'eigenstates':1e-6})
    c.calculate(s)
    c.write(fname)
    c.write(fwfname, 'all')

# full information
c = GPAW(fwfname, txt=txt)
E_PBE = c.get_potential_energy()
try: # number of iterations needed in restart
    niter_PBE = c.get_number_of_iterations()
except: pass
dE = c.get_xc_difference({'name': 'TPSS', 'stencil': 1})
E_1 = E_PBE + dE
print("E PBE, TPSS=", E_PBE, E_1)

# no wfs
c = GPAW(fname, txt=txt)
E_PBE_no_wfs = c.get_potential_energy()
try: # number of iterations needed in restart
    niter_PBE_no_wfs = c.get_number_of_iterations()
except: pass
dE = c.get_xc_difference({'name': 'TPSS', 'stencil': 1})
E_2 = E_PBE_no_wfs + dE
print("E PBE, TPSS=", E_PBE_no_wfs, E_2)

print("diff=", E_1 - E_2)
assert abs(E_1 - E_2) < 0.005

energy_tolerance = 0.002
equal(E_PBE, -5.341, energy_tolerance)
equal(E_PBE_no_wfs, -5.33901, energy_tolerance)
equal(E_1, -5.57685, energy_tolerance)
equal(E_2, -5.57685, energy_tolerance)



from __future__ import print_function
# This tests calculates the force on the atoms of a
# slightly distorted Silicon primitive cell.
#
# If the test fails, set the fd boolean below to enable a (costly) finite
# difference check.

import numpy as np
from ase import Atoms
from gpaw import GPAW
from gpaw.atom.basis import BasisMaker
from gpaw.test import equal

sibasis = BasisMaker('Si').generate(2, 1, energysplit=0.3, tailnorm=0.03**.5)
basis = {'Si' : sibasis}

a = 5.475
system = Atoms(symbols='Si2', pbc=True,
               cell=0.5 * a * np.array([(1, 1, 0),
                                        (1, 0, 1),
                                        (0, 1, 1)]),
               scaled_positions=[(0.0, 0.0, 0.0),
                                 (0.23, 0.23, 0.23)])

calc = GPAW(h=0.2,
            mode='lcao',
            basis=basis,
            kpts=(2,2,2),
            convergence={'density':1e-5, 'energy': 1e-6}
            )
system.set_calculator(calc)

F_ac = system.get_forces()

# Previous FD result, generated by disabled code below
F_ac_ref = np.array(
    [[-1.3967114867039498, -1.3967115816022613, -1.396711581510779],
     [1.397400325299003, 1.3974003410455182, 1.3974003410801572]])

err_ac = np.abs(F_ac - F_ac_ref)
err = err_ac.max()

print('Force')
print(F_ac)
print()
print('Reference result')
print(F_ac_ref)
print()
print('Error')
print(err_ac)
print()
print('Max error')
print(err)

# ASE uses dx = [+|-] 0.001 by default,
# error should be around 2e-3.  In fact 4e-3 would probably be acceptable


equal(err, 0, 6e-3)

# Set boolean to run new FD check
fd = False

if fd:
    from ase.calculators.test import numeric_force
    calc.set(symmetry='off')
    F_ac_fd = [[numeric_force(system, a, i) for i in range(3)]
               for a in range(2)]
    print('Self-consistent forces')
    print(F_ac)
    print('FD')
    print(F_ac_fd)
    print(repr(F_ac_fd))
    print(F_ac - F_ac_fd, np.abs(F_ac - F_ac_fd).max())

    assert np.abs(F_ac - F_ac_fd).max() < 4e-3



from math import pi

from ase import Atom, Atoms
from ase.parallel import parprint

from gpaw import GPAW
from gpaw.test import equal
from gpaw.analyse.expandyl import AngularIntegral, ExpandYl

fname = 'H2.gpw'
donot = ''
donot = 'donot'
try:
    calc = GPAW(fname + donot, txt=None)
    H2 = calc.get_atoms()
    calc.converge_wave_functions()
except:
    R=0.7 # approx. experimental bond length
    a = 2.
    c = 3.
    H2 = Atoms([Atom('H', (a/2, a/2, (c-R)/2)),
                Atom('H', (a/2, a/2, (c+R)/2))],
               cell=(a,a,c), pbc=True)
    calc = GPAW(gpts=(12, 12, 16), nbands=2, kpts=(1,1,2),
                convergence={'eigenstates':1.e-6},
                txt=None,
                )
    H2.set_calculator(calc)
    H2.get_potential_energy()
    if not donot:
        calc.write(fname)

# Check that a / h = 10 is rounded up to 12 as always:
assert (calc.wfs.gd.N_c == (12, 12, 16)).all()

############ AngularIntegral

gd = calc.density.gd
ai = AngularIntegral(H2.positions.mean(0), calc.wfs.gd, Rmax=1.5)
unity_g = gd.zeros() + 1.
average_R = ai.average(unity_g)
integral_R = ai.integrate(unity_g)
for V, average, integral, R, Rm in zip(ai.V_R, average_R, integral_R, 
                                       ai.radii(), ai.radii('mean')):
    if V > 0:
        equal(average, 1, 1.e-9)
        equal(integral / (4 * pi * Rm**2), 1, 0.61)
        equal(Rm / R, 1, 0.61)

############ ExpandYl

yl = ExpandYl(H2.positions.mean(0), calc.wfs.gd, Rmax=1.5)

def max_index(l):
    mi = 0
    limax = l[0]
    for i, li in enumerate(l):
        if limax < li:
            limax = li
            mi = i
    return mi

# check numbers
for n in [0,1]:
    #gl, w = yl.expand(calc.get_pseudo_wave_function(band=n))
    gl, w = yl.expand(calc.wfs.kpt_u[0].psit_nG[n])
    parprint('max_index(gl), n=', max_index(gl), n)
    assert(max_index(gl) == n)

# io
fname = 'expandyl.dat'
yl.to_file(calc, fname)



from __future__ import division
from math import sqrt
import numpy as np
from ase import Atoms
from gpaw import GPAW, PW, restart
from gpaw.test import equal

d = 3.0
atoms = Atoms('Na3',
              positions=[(0, 0, 0),
                         (0, 0, d),
                         (0, d * sqrt(3 / 4), d / 2)],
              magmoms=[1.0, 1.0, 1.0],
              cell=(3.5, 3.5, 4 + 2 / 3),
              pbc=True)


def test(atoms):
    e0 = atoms.get_potential_energy()
    f0 = atoms.get_forces()
    m0 = atoms.get_magnetic_moments()
    eig00 = atoms.calc.get_eigenvalues(spin=0)
    eig01 = atoms.calc.get_eigenvalues(spin=1)
    wf0 = atoms.calc.get_pseudo_wave_function(band=1)
    # Write the restart file(s):
    atoms.calc.write('tmp1.gpw')
    atoms.calc.write('tmp2.gpw', 'all')

    # Try restarting:
    atoms, calc = restart('tmp2.gpw', txt=None)
    wf1 = calc.get_pseudo_wave_function(band=1)
    e1 = atoms.get_potential_energy()
    f1 = atoms.get_forces()
    m1 = atoms.get_magnetic_moments()
    eig10 = calc.get_eigenvalues(spin=0)
    eig11 = calc.get_eigenvalues(spin=1)
    print(e0, e1)
    equal(e0, e1, 1e-10)
    print(f0, f1)
    for ff0, ff1 in zip(f0, f1):
        err = np.linalg.norm(ff0 - ff1)
        assert err <= 1e-10
    print(m0, m1)
    for mm0, mm1 in zip(m0, m1):
        equal(mm0, mm1, 1e-10)
    print('A', eig00, eig10)
    for eig0, eig1 in zip(eig00, eig10):
        equal(eig0, eig1, 1e-10)
    print('B', eig01, eig11)
    for eig0, eig1 in zip(eig01, eig11):
        equal(eig0, eig1, 1e-10)
    equal(abs(wf1 - wf0).max(), 0, 1e-14)

    # Check that after restart everything is writable
    calc.write('tmp3.gpw')
    calc.write('tmp4.gpw', 'all')


# Only a short, non-converged calcuation
conv = {'eigenstates': 1.24, 'energy': 2e-1, 'density': 1e-1}

for kwargs in [{'mode': PW(200)},
               {'h': 0.30}]:
    atoms.calc = GPAW(nbands=3,
                      setups={'Na': '1'},
                      convergence=conv,
                      txt=None,
                      **kwargs)
    test(atoms)



# for n > 4 (?)
# when run with gpaw-python built with MKL matrices b and c have ~zero elements
# running with python (uses _gpaw.so) gives right (non-zero) results!

import numpy as np

n = 5 # works for n = 4
a1 = np.eye(n) + 1.j
a2 = a1 + 1.j

from gpaw.utilities.blas import gemm
b = np.zeros((n, n), dtype=complex)

c = np.dot(a2, a1)

gemm(1.0, a1, a2, 0.0, b)

thresh = 1.0e-7
ref_max_value = -9.0

#print b
#print c
numpy_dot = np.max(b).real
gpaw_gemm = np.max(c).real
#print gpaw_gemm
assert abs(gpaw_gemm-numpy_dot) < thresh, (gpaw_gemm, numpy_dot, thresh)
assert abs(gpaw_gemm-ref_max_value) < thresh, (gpaw_gemm, ref_max_value, thresh)



from __future__ import print_function
import numpy as np

from gpaw import __version__ as version
from gpaw.mpi import world
from gpaw.tddft.units import (au_to_as, au_to_fs, au_to_eV)
from gpaw.tddft.folding import FoldedFrequencies
from gpaw.tddft.folding import Folding


def read_dipole_moment_file(fname, remove_duplicates=True):
    def parse_kick_line(line):
        kick_str_v = line.split('[', 1)[1].split(']', 1)[0].split(',')
        kick_v = np.array([float(x) for x in kick_str_v])
        data_i = line.split('Time =')
        if len(data_i) == 1:
            time = 0.0
        else:
            time = float(data_i[1])
        return kick_v, time

    # Search kicks
    kick_i = []
    with open(fname, 'r') as f:
        for line in f:
            if line.startswith('# Kick'):
                kick_v, time = parse_kick_line(line)
                kick_i.append({'strength_v': kick_v, 'time': time})

    # Read data
    data_tj = np.loadtxt(fname)
    time_t = data_tj[:, 0]
    norm_t = data_tj[:, 1]
    dm_tv = data_tj[:, 2:]

    # Remove duplicates due to abruptly stopped and restarted calculation
    if remove_duplicates:
        flt_t = np.ones_like(time_t, dtype=bool)
        maxtime = time_t[0]
        for t in range(1, len(time_t)):
            if time_t[t] > maxtime:
                maxtime = time_t[t]
            else:
                flt_t[t] = False

        time_t = time_t[flt_t]
        norm_t = norm_t[flt_t]
        dm_tv = dm_tv[flt_t]

        ndup = len(flt_t) - flt_t.sum()
        if ndup > 0:
            print('Removed %d duplicates' % ndup)

    return kick_i, time_t, norm_t, dm_tv


def calculate_polarizability(data, foldedfrequencies):
    kick_v, time_t, dm_tv = data
    ff = foldedfrequencies
    omega_w = ff.frequencies
    envelope = ff.folding.envelope

    time_t = time_t - time_t[0]
    dt_t = np.insert(time_t[1:] - time_t[:-1], 0, 0.0)
    dm_tv = dm_tv[:] - dm_tv[0]

    kick_magnitude = np.sum(kick_v**2)

    Nw = len(omega_w)
    alpha_wv = np.zeros((Nw, 3), dtype=complex)
    f_wt = np.exp(1.0j * np.outer(omega_w, time_t))
    dm_vt = np.swapaxes(dm_tv, 0, 1)
    alpha_wv = np.tensordot(f_wt, dt_t * envelope(time_t) * dm_vt, axes=(1, 1))
    alpha_wv *= kick_v / kick_magnitude
    return alpha_wv


def calculate_photoabsorption(data, foldedfrequencies):
    omega_w = foldedfrequencies.frequencies
    alpha_wv = calculate_polarizability(data, foldedfrequencies)
    abs_wv = 2 / np.pi * omega_w[:, np.newaxis] * alpha_wv.imag
    return abs_wv


def write_spectrum(dipole_moment_file, spectrum_file,
                   folding, width, e_min, e_max, delta_e,
                   title, symbol, calculate):
    def str_list(v_i, fmt='%g'):
        return '[%s]' % ', '.join(map(lambda v: fmt % v, v_i))

    r = read_dipole_moment_file(dipole_moment_file)
    kick_i, time_t, norm_t, dm_tv = r
    dt_t = time_t[1:] - time_t[:-1]

    if len(kick_i) > 1:
        raise RuntimeError('Multiple kicks in %s' % dipole_moment_file)
    kick = kick_i[0]
    kick_v = kick['strength_v']
    kick_time = kick['time']

    # Discard times before kick
    flt_t = time_t > (kick_time - 0.5 * dt_t.min())
    time_t = time_t[flt_t]
    norm_t = norm_t[flt_t]
    dm_tv = dm_tv[flt_t]
    dt_t = time_t[1:] - time_t[:-1]

    freqs = np.arange(e_min, e_max + 0.5 * delta_e, delta_e)
    folding = Folding(folding, width)
    ff = FoldedFrequencies(freqs, folding)
    omega_w = ff.frequencies
    spec_wv = calculate((kick_v, time_t, dm_tv), ff)

    # Write spectrum file header
    with open(spectrum_file, 'w') as f:
        def w(s):
            f.write('%s\n' % s)

        w('# %s spectrum from real-time propagation' % title)
        w('# GPAW version: %s' % version)
        w('# Total time = %.4f fs, Time steps = %s as' %
          (dt_t.sum() * au_to_fs,
           str_list(np.unique(np.around(dt_t, 6)) * au_to_as, '%.4f')))
        w('# Kick = %s' % str_list(kick_v))
        w('# %sian folding, Width = %.4f eV = %lf Hartree'
          ' <=> FWHM = %lf eV' %
          (folding.folding, folding.width * au_to_eV, folding.width,
           folding.fwhm * au_to_eV))

        col_i = []
        data_iw = [omega_w * au_to_eV]
        for v in range(len(kick_v)):
            h = '%s_%s' % (symbol, 'xyz'[v])
            if spec_wv.dtype == complex:
                col_i.append('Re[%s]' % h)
                data_iw.append(spec_wv[:, v].real)
                col_i.append('Im[%s]' % h)
                data_iw.append(spec_wv[:, v].imag)
            else:
                col_i.append(h)
                data_iw.append(spec_wv[:, v])

        w('# %10s %s' % ('om (eV)', ' '.join(['%20s' % s for s in col_i])))

    # Write spectrum file data
    with open(spectrum_file, 'ab') as f:
        np.savetxt(f, np.array(data_iw).T,
                   fmt='%12.6lf' + (' %20.10le' * len(col_i)))

    return folding.envelope(time_t[-1])


def photoabsorption_spectrum(dipole_moment_file, spectrum_file,
                             folding='Gauss', width=0.2123,
                             e_min=0.0, e_max=30.0, delta_e=0.05):
    """Calculates photoabsorption spectrum from the time-dependent
    dipole moment.

    Parameters:

    dipole_moment_file: string
        Name of the time-dependent dipole moment file from which
        the spectrum is calculated
    spectrum_file: string
        Name of the spectrum file
    folding: 'Gauss' or 'Lorentz'
        Whether to use Gaussian or Lorentzian folding
    width: float
        Width of the Gaussian (sigma) or Lorentzian (Gamma)
        Gaussian =     1/(sigma sqrt(2pi)) exp(-(1/2)(omega/sigma)^2)
        Lorentzian =  (1/pi) (1/2) Gamma / [omega^2 + ((1/2) Gamma)^2]
    e_min: float
        Minimum energy shown in the spectrum (eV)
    e_max: float
        Maximum energy shown in the spectrum (eV)
    delta_e: float
        Energy resolution (eV)
    """
    if world.rank == 0:
        print('Calculating photoabsorption spectrum from file "%s"'
              % dipole_moment_file)

        def calculate(*args):
            return calculate_photoabsorption(*args) / au_to_eV
        sinc = write_spectrum(dipole_moment_file, spectrum_file,
                              folding, width, e_min, e_max, delta_e,
                              'Photoabsorption', 'S', calculate)
        print('Sinc contamination %.8f' % sinc)
        print('Calculated photoabsorption spectrum saved to file "%s"'
              % spectrum_file)


def polarizability_spectrum(dipole_moment_file, spectrum_file,
                            folding='Gauss', width=0.2123,
                            e_min=0.0, e_max=30.0, delta_e=0.05):
    """Calculates polarizability spectrum from the time-dependent
    dipole moment.

    Parameters:

    dipole_moment_file: string
        Name of the time-dependent dipole moment file from which
        the spectrum is calculated
    spectrum_file: string
        Name of the spectrum file
    folding: 'Gauss' or 'Lorentz'
        Whether to use Gaussian or Lorentzian folding
    width: float
        Width of the Gaussian (sigma) or Lorentzian (Gamma)
        Gaussian =     1/(sigma sqrt(2pi)) exp(-(1/2)(omega/sigma)^2)
        Lorentzian =  (1/pi) (1/2) Gamma / [omega^2 + ((1/2) Gamma)^2]
    e_min: float
        Minimum energy shown in the spectrum (eV)
    e_max: float
        Maximum energy shown in the spectrum (eV)
    delta_e: float
        Energy resolution (eV)
    """
    if world.rank == 0:
        print('Calculating polarizability spectrum from file "%s"'
              % dipole_moment_file)

        def calculate(*args):
            return calculate_polarizability(*args) / au_to_eV**2
        sinc = write_spectrum(dipole_moment_file, spectrum_file,
                              folding, width, e_min, e_max, delta_e,
                              'Polarizability', 'alpha', calculate)
        print('Sinc contamination %.8f' % sinc)
        print('Calculated polarizability spectrum saved to file "%s"'
              % spectrum_file)



from __future__ import division, print_function

import sys
from math import pi
import pickle

import numpy as np

from ase.utils.timing import timer
from ase.units import Hartree
from ase.dft.kpoints import monkhorst_pack

import gpaw.mpi as mpi
from gpaw.kpt_descriptor import KPointDescriptor
from gpaw.response.chi0 import HilbertTransform, frequency_grid
from gpaw.response.pair import PairDensity
from gpaw.wavefunctions.pw import PWDescriptor
from gpaw.xc.exx import select_kpts


class GWQEHCorrection(PairDensity):
    def __init__(self, calc, gwfile, filename=None, kpts=[0], bands=None,
                 structure=None, d=None, layer=0, 
                 dW_qw=None, qqeh=None, wqeh=None,
                 txt=sys.stdout, world=mpi.world, domega0=0.025, 
                 omega2=10.0, eta=0.1, include_q0=True, metal=False): 
        """ 
        Class for calculating quasiparticle energies of van der Waals
        heterostructures using the GW approximation for the self-energy. 
        The quasiparticle energy correction due to increased screening from
        surrounding layers is obtained from the QEH model.
        Parameters:
        
        calc: str or PAW object
            GPAW calculator object or filename of saved calculator object.
        gwfile: str
            name of gw results file from the monolayer calculation
        filename: str
            filename for gwqeh output
        kpts: list
            List of indices of sthe IBZ k-points to calculate the quasi
            particle energies for. Set to [0] by default since the QP 
            correction is generally the same for all k. 
        bands: tuple
            Range of band indices, like (n1, n2+1), to calculate the quasi
            particle energies for. Note that the second band index is not
            included. Should be the same as used for the GW calculation.
        structure: list of str
            Heterostructure set up. Each entry should consist of number of
            layers + chemical formula.
            For example: ['3H-MoS2', graphene', '10H-WS2'] gives 3 layers of
            H-MoS2, 1 layer of graphene and 10 layers of H-WS2.
            The name of the layers should correspond to building block files:
            "<name>-chi.npz" in the local repository.
        d: array of floats
            Interlayer distances for neighboring layers in Ang.
            Length of array = number of layers - 1
        layer: int
            index of layer to calculate QP correction for. 
        dW_qw: 2D array of floats dimension q X w
            Change in screened interaction. Should be set to None to calculate 
            dW directly from buildingblocks.
        qqeh: array of floats
            q-grid used for dW_qw (only needed if dW is given by hand).
        wqeh: array of floats
            w-grid used for dW_qw. So far this have to be the same as for the 
            GWQEH calculation.  (only needed if dW is given by hand).
        domega0: float
            Minimum frequency step (in eV) used in the generation of the non-
            linear frequency grid.
        omega2: float
            Control parameter for the non-linear frequency grid, equal to the
            frequency where the grid spacing has doubled in size.
        eta: float
            Broadening parameter.
        include_q0: bool
            include q=0 in W or not. if True an integral arround q=0 is
            performed, if False the q=0 contribution is set to zero. 
        metal: bool
            If True, the point at q=0 is omitted when averaging the screened
            potential close to q=0. 
        """
        
        self.gwfile = gwfile

        self.inputcalc = calc
        # Set low ecut in order to use PairDensity object since only
        # G=0 is needed.
        self.ecut = 1.
        PairDensity.__init__(self, calc, ecut=self.ecut, world=world,
                             txt=filename + '.txt')    
    
        self.filename = filename
        self.ecut /= Hartree
        self.eta = eta / Hartree
        self.domega0 = domega0 / Hartree
        self.omega2 = omega2 / Hartree

        self.kpts = list(select_kpts(kpts, self.calc))

        if bands is None:
            bands = [0, self.nocc2]
            
        self.bands = bands

        b1, b2 = bands
        self.shape = shape = (self.calc.wfs.nspins, len(self.kpts), b2 - b1)
        self.eps_sin = np.empty(shape)     # KS-eigenvalues
        self.f_sin = np.empty(shape)       # occupation numbers
        self.sigma_sin = np.zeros(shape)   # self-energies
        self.dsigma_sin = np.zeros(shape)  # derivatives of self-energies
        self.Z_sin = None                  # renormalization factors
        self.qp_sin = None
        self.Qp_sin = None
        
        self.ecutnb = 150 / Hartree
        vol = abs(np.linalg.det(self.calc.wfs.gd.cell_cv))
        self.vol = vol
        self.nbands = min(self.calc.get_number_of_bands(),
                          int(vol * (self.ecutnb)**1.5 * 2**0.5 / 3 / pi**2))

        self.nspins = self.calc.wfs.nspins

        kd = self.calc.wfs.kd

        self.mysKn1n2 = None  # my (s, K, n1, n2) indices
        self.distribute_k_points_and_bands(b1, b2, kd.ibz2bz_k[self.kpts])
        
        # Find q-vectors and weights in the IBZ:
        assert -1 not in kd.bz2bz_ks
        offset_c = 0.5 * ((kd.N_c + 1) % 2) / kd.N_c
        bzq_qc = monkhorst_pack(kd.N_c) + offset_c
        self.qd = KPointDescriptor(bzq_qc)
        self.qd.set_symmetry(self.calc.atoms, kd.symmetry)

        # frequency grid
        omax = self.find_maximum_frequency()
        self.omega_w = frequency_grid(self.domega0, self.omega2, omax)
        self.nw = len(self.omega_w)
        self.wsize = 2 * self.nw

        # Calculate screened potential of Heterostructure
        if dW_qw is None:
            try:
                data = np.load(filename + "_dW_qw.npz")
                self.qqeh = data['qqeh']
                self.wqeh = data['wqeh']
                dW_qw = data['dW_qw']
            except IOError:
                dW_qw = self.calculate_W_QEH(structure, d, layer)
        else:
            self.qqeh = qqeh
            self.wqeh = None  # wqeh
        
        self.dW_qw = self.get_W_on_grid(dW_qw, include_q0=include_q0,
                                        metal=metal)

        assert self.nw == self.dW_qw.shape[1], \
            ('Frequency grids doesnt match!')

        self.htp = HilbertTransform(self.omega_w, self.eta, gw=True)
        self.htm = HilbertTransform(self.omega_w, -self.eta, gw=True)

        self.complete = False
        self.nq = 0
        if self.load_state_file():
            if self.complete:
                print('Self-energy loaded from file', file=self.fd)
        
    def calculate_QEH(self):
        print('Calculating QEH self-energy contribution', file=self.fd)

        kd = self.calc.wfs.kd

        # Reset calculation
        self.sigma_sin = np.zeros(self.shape)   # self-energies
        self.dsigma_sin = np.zeros(self.shape)  # derivatives of self-energies

        # Get KS eigenvalues and occupation numbers:
        b1, b2 = self.bands
        nibzk = self.calc.wfs.kd.nibzkpts
        for i, k in enumerate(self.kpts):
            for s in range(self.nspins):
                u = s * nibzk + k
                kpt = self.calc.wfs.kpt_u[u]
                self.eps_sin[s, i] = kpt.eps_n[b1:b2]
                self.f_sin[s, i] = kpt.f_n[b1:b2] / kpt.weight

        # My part of the states we want to calculate QP-energies for:
        mykpts = [self.get_k_point(s, K, n1, n2)
                  for s, K, n1, n2 in self.mysKn1n2]

        Nq = len((self.qd.ibzk_kc))
        for iq, q_c in enumerate(self.qd.ibzk_kc):
            self.nq = iq
            nq = iq
            self.save_state_file()
     
            qcstr = '(' + ', '.join(['%.3f' % x for x in q_c]) + ')'
            print('Calculating contribution from IBZ q-point #%d/%d q_c=%s'
                  % (nq, Nq, qcstr), file=self.fd)
            

            # Screened potential
            dW_w = self.dW_qw[nq]
            dW_w = dW_w[:, np.newaxis, np.newaxis]
            L = abs(self.calc.wfs.gd.cell_cv[2, 2])
            dW_w *= L 

            nw = self.nw

            Wpm_w = np.zeros([2 * nw, 1, 1], dtype=complex)
            Wpm_w[:nw] = dW_w
            Wpm_w[nw:] = Wpm_w[0:nw]
            
            with self.timer('Hilbert transform'):
                self.htp(Wpm_w[:nw])
                self.htm(Wpm_w[nw:])

            qd = KPointDescriptor([q_c])
            pd0 = PWDescriptor(self.ecut, self.calc.wfs.gd, complex, qd)

            # modify pd0 by hand - only G=0 component is needed
            pd0.G_Qv = np.array([1e-17, 1e-17, 1e-17])[np.newaxis, :] 
            pd0.Q_qG = [np.array([0], dtype='int32')]
            pd0.ngmax = 1
            G_Gv = pd0.get_reciprocal_vectors()
        
            self.Q_aGii = self.initialize_paw_corrections(pd0)

            # Loop over all k-points in the BZ and find those that are related
            # to the current IBZ k-point by symmetry
            Q1 = self.qd.ibz2bz_k[iq]
            Q2s = set()
            for s, Q2 in enumerate(self.qd.bz2bz_ks[Q1]):
                if Q2 >= 0 and Q2 not in Q2s:
                    Q2s.add(Q2)
                    
            for Q2 in Q2s:
                s = self.qd.sym_k[Q2]
                self.s = s
                U_cc = self.qd.symmetry.op_scc[s]
                time_reversal = self.qd.time_reversal_k[Q2]
                self.sign = 1 - 2 * time_reversal
                Q_c = self.qd.bzk_kc[Q2]
                d_c = self.sign * np.dot(U_cc, q_c) - Q_c
                assert np.allclose(d_c.round(), d_c)
              
                for u1, kpt1 in enumerate(mykpts):
                    K2 = kd.find_k_plus_q(Q_c, [kpt1.K])[0] 
                    kpt2 = self.get_k_point(kpt1.s, K2, 0, self.nbands, 
                                            block=True)
                    k1 = kd.bz2ibz_k[kpt1.K]
                    i = self.kpts.index(k1)

                    N_c = pd0.gd.N_c
                    i_cG = self.sign * np.dot(U_cc, 
                                              np.unravel_index(pd0.Q_qG[0], 
                                                               N_c))

                    k1_c = kd.bzk_kc[kpt1.K]
                    k2_c = kd.bzk_kc[K2]
                    # This is the q that connects K1 and K2 in the 1st BZ
                    q1_c = k2_c - k1_c

                    # G-vector that connects the full Q_c with q1_c
                    shift1_c = q1_c - self.sign * np.dot(U_cc, q_c)
                    assert np.allclose(shift1_c.round(), shift1_c)
                    shift1_c = shift1_c.round().astype(int)
                    shift_c = kpt1.shift_c - kpt2.shift_c - shift1_c
                    I_G = np.ravel_multi_index(i_cG + shift_c[:, None], 
                                               N_c, 'wrap') 
                    pos_av = np.dot(self.spos_ac, pd0.gd.cell_cv)
                    M_vv = np.dot(pd0.gd.cell_cv.T,
                                  np.dot(U_cc.T,
                                         np.linalg.inv(pd0.gd.cell_cv).T))
                    Q_aGii = []
                    for a, Q_Gii in enumerate(self.Q_aGii):
                        x_G = np.exp(1j * np.dot(G_Gv, (pos_av[a] -
                                                        np.dot(M_vv, 
                                                               pos_av[a]))))
                        U_ii = self.calc.wfs.setups[a].R_sii[self.s]
                        Q_Gii = np.dot(np.dot(U_ii, Q_Gii * x_G[:, None, 
                                                                   None]),
                                       U_ii.T).transpose(1, 0, 2)
                        if self.sign == -1:
                            Q_Gii = Q_Gii.conj()
                        Q_aGii.append(Q_Gii)

                    for n in range(kpt1.n2 - kpt1.n1):
                        ut1cc_R = kpt1.ut_nR[n].conj()
                        eps1 = kpt1.eps_n[n]
                        C1_aGi = [np.dot(Qa_Gii, P1_ni[n].conj())
                                  for Qa_Gii, P1_ni in zip(Q_aGii, kpt1.P_ani)]

                        n_mG = self.calculate_pair_densities(ut1cc_R, C1_aGi, 
                                                             kpt2, pd0, I_G)
                        if self.sign == 1:
                            n_mG = n_mG.conj()
                                    
                        f_m = kpt2.f_n
                        deps_m = eps1 - kpt2.eps_n
                        sigma, dsigma = self.calculate_sigma(n_mG, deps_m, 
                                                             f_m, Wpm_w)
                        nn = kpt1.n1 + n - self.bands[0]
                        self.sigma_sin[kpt1.s, i, nn] += sigma
                        self.dsigma_sin[kpt1.s, i, nn] += dsigma
                       
        self.world.sum(self.sigma_sin)
        self.world.sum(self.dsigma_sin)

        self.complete = True
        self.save_state_file()
    
        return self.sigma_sin, self.dsigma_sin

    def calculate_qp_correction(self):

        if self.complete:
            print('Self-energy loaded from file', file=self.fd)
        else:
            self.calculate_QEH()  

        # Need GW result for renormalization factor
        b1, b2 = self.bands
        gwdata = pickle.load(open(self.gwfile, 'rb'))
        self.dsigmagw_sin = gwdata['dsigma']
        self.qpgw_sin = gwdata['qp'] / Hartree
        nk = self.qpgw_sin.shape[1]
        if not self.sigma_sin.shape[1] == nk:
            self.sigma_sin = np.repeat(self.sigma_sin[:, :1, :], nk, axis=1)
            self.dsigma_sin = np.repeat(self.dsigma_sin[:, :1, :], nk, axis=1)
        self.Z_sin = 1. / (1 - self.dsigma_sin - self.dsigmagw_sin)
        self.qp_sin = self.Z_sin * self.sigma_sin
        
        return self.qp_sin * Hartree
    
    def calculate_qp_energies(self):
        # calculate 
        qp_sin = self.calculate_qp_correction() / Hartree
        self.Qp_sin = self.qpgw_sin + qp_sin
        self.save_state_file()
        return self.Qp_sin * Hartree

    @timer('Sigma')
    def calculate_sigma(self, n_mG, deps_m, f_m, W_wGG):
        """Calculates a contribution to the self-energy and its derivative for
        a given (k, k-q)-pair from its corresponding pair-density and
        energy."""
        o_m = abs(deps_m)
        # Add small number to avoid zeros for degenerate states:
        sgn_m = np.sign(deps_m + 1e-15)
        
        # Pick +i*eta or -i*eta:
        s_m = (1 + sgn_m * np.sign(0.5 - f_m)).astype(int) // 2
        comm = self.blockcomm
        nw = len(self.omega_w)
        nG = n_mG.shape[1]
        mynG = (nG + comm.size - 1) // comm.size
        Ga = min(comm.rank * mynG, nG)
        Gb = min(Ga + mynG, nG)
        beta = (2**0.5 - 1) * self.domega0 / self.omega2
        w_m = (o_m / (self.domega0 + beta * o_m)).astype(int)
        o1_m = self.omega_w[w_m]
        o2_m = self.omega_w[w_m + 1]
        
        x = 1.0 / (self.qd.nbzkpts * 2 * pi * self.vol)
        sigma = 0.0
        dsigma = 0.0
        # Performing frequency integration
        for o, o1, o2, sgn, s, w, n_G in zip(o_m, o1_m, o2_m,
                                             sgn_m, s_m, w_m, n_mG):
        
            C1_GG = W_wGG[s*nw + w]
            C2_GG = W_wGG[s*nw + w + 1]
            p = x * sgn
            myn_G = n_G[Ga:Gb]
            sigma1 = p * np.dot(np.dot(myn_G, C1_GG), n_G.conj()).imag
            sigma2 = p * np.dot(np.dot(myn_G, C2_GG), n_G.conj()).imag
            sigma += ((o - o1) * sigma2 + (o2 - o) * sigma1) / (o2 - o1)
            dsigma += sgn * (sigma2 - sigma1) / (o2 - o1)

        return sigma, dsigma

    def save_state_file(self, q=0):
        data = {'kpts': self.kpts,
                'bands': self.bands,
                'nbands': self.nbands,
                'last_q': self.nq,
                'complete': self.complete,
                'sigma_sin': self.sigma_sin,
                'dsigma_sin': self.dsigma_sin,
                'qp_sin': self.qp_sin,
                'Qp_sin': self.Qp_sin}
        if self.world.rank == 0:
            np.savez(self.filename + '_qeh.npz',
                     **data)

    def load_state_file(self):
        try:
            data = np.load(self.filename + '_qeh.npz')
        except IOError:
            return False
        else:
            if (data['kpts'] == self.kpts and
                data['bands'] == self.bands and
                data['nbands'] == self.nbands):
                self.nq = data['last_q']
                self.complete = data['complete']
                self.complete = data['complete']
                self.sigma_sin = data['sigma_sin']
                self.dsigma_sin = data['dsigma_sin']
                return True
            else:
                return False    

    def get_W_on_grid(self, dW_qw, include_q0=True, metal=False):
        """This function transforms the screened potential W(q,w) to the 
        (q,w)-grid of the GW calculation. Also, W is integrated over
        a region around q=0 if include_q0 is set to True."""
    
        q_cs = self.qd.ibzk_kc

        rcell_cv = 2 * pi * np.linalg.inv(self.calc.wfs.gd.cell_cv).T
        q_vs = np.dot(q_cs, rcell_cv)
        q_grid = (q_vs**2).sum(axis=1)**0.5
        self.q_grid = q_grid
        w_grid = self.omega_w

        wqeh = self.wqeh  # w_grid.copy() # self.qeh
        qqeh = self.qqeh
        sortqeh = np.argsort(qqeh)
        qqeh = qqeh[sortqeh]
        dW_qw = dW_qw[sortqeh] 

        sort = np.argsort(q_grid)
        isort = np.argsort(sort)
        if metal and np.isclose(qqeh[0], 0):
            """We don't have the right q=0 limit for metals  and semi-metals.
            -> Point should be omitted from interpolation"""
            qqeh = qqeh[1:]
            dW_qw = dW_qw[1:]
            sort = sort[1:]
            
        from scipy.interpolate import RectBivariateSpline
        yr = RectBivariateSpline(qqeh, wqeh, dW_qw.real, s=0)
        yi = RectBivariateSpline(qqeh, wqeh, dW_qw.imag, s=0)

        dWgw_qw = yr(q_grid[sort], w_grid) + 1j * yi(q_grid[sort], w_grid)
        dW_qw = yr(qqeh, w_grid) + 1j * yi(qqeh, w_grid)
        
        if metal:
            # Interpolation is done -> put back zeros at q=0
            dWgw_qw = np.insert(dWgw_qw, 0, 0, axis=0)
            qqeh = np.insert(qqeh, 0, 0)
            dW_qw = np.insert(dW_qw, 0, 0, axis=0)
            q_cut = q_grid[sort][0] / 2.
        else:
            q_cut = q_grid[sort][1] / 2.
            
        q0 = np.array([q for q in qqeh if q <= q_cut])
        if len(q0) > 1:  # Integrate arround q=0 
            vol = np.pi * (q0[-1] + q0[1] / 2.)**2 
            if np.isclose(q0[0], 0):
                weight0 = np.pi * (q0[1] / 2.)**2 / vol
                c = (1 - weight0) / np.sum(q0)
                weights = c * q0        
                weights[0] = weight0
            else:
                c = 1 / np.sum(q0)
                weights = c * q0        

            dWgw_qw[0] = (np.repeat(weights[:, np.newaxis], len(w_grid), 
                                    axis=1) * dW_qw[:len(q0)]).sum(axis=0)
        
        if not include_q0:  # Omit q=0 contrinution completely.
            dWgw_qw[0] = 0.0
             
        dWgw_qw = dWgw_qw[isort]  # Put dW back on native grid.
        return dWgw_qw

    def calculate_W_QEH(self, structure, d, layer=0):
        from gpaw.response.qeh import Heterostructure, expand_layers

        structure = expand_layers(structure)
        self.w_grid = self.omega_w
        wmax = self.w_grid[-1]
        # qmax = (self.q_grid).max()

        # Single layer
        s = (np.insert(d, 0, d[0]) +
             np.append(d, d[-1])) / 2.
        d0 = s[layer]
        HS0 = Heterostructure(structure=[structure[layer]], 
                              d=[],
                              d0=d0,
                              wmax=wmax * Hartree,
                              # qmax=qmax / Bohr
                              ) 

        W0_qw = HS0.get_screened_potential()
        
        # Full heterostructure
      
        HS = Heterostructure(structure=structure, d=d,
                             wmax=wmax * Hartree,
                             # qmax=qmax / Bohr
                             ) 
        W_qw = HS.get_screened_potential(layer=layer)
    
        # Difference in screened potential:
        dW_qw = W_qw - W0_qw
        self.wqeh = HS.frequencies
        self.qqeh = HS.q_abs

        if self.world.rank == 0:
            data = {'qqeh': self.qqeh,
                    'wqeh': self.wqeh,
                    'dW_qw': dW_qw}
            np.savez(self.filename + "_dW_qw.npz",
                     **data)

        return dW_qw

    def find_maximum_frequency(self):
        self.epsmin = 10000.0
        self.epsmax = -10000.0
        for kpt in self.calc.wfs.kpt_u:
            self.epsmin = min(self.epsmin, kpt.eps_n[0])
            self.epsmax = max(self.epsmax, kpt.eps_n[self.nbands - 1])
            
        print('Minimum eigenvalue: %10.3f eV' % (self.epsmin * Hartree),
              file=self.fd)
        print('Maximum eigenvalue: %10.3f eV' % (self.epsmax * Hartree),
              file=self.fd)

        return self.epsmax - self.epsmin



#!/usr/bin/env python3
"""Bash completion for ase.

Put this in your .bashrc::

    complete -o default -C /path/to/gpaw/cli/complete.py gpaw

or run::

    $ gpaw completion [-0] [<your-bash-file>]

"""

from __future__ import print_function
import os
import sys
from glob import glob


def match(word, *suffixes):
    return [w for w in glob(word + '*')
            if any(w.endswith(suffix) for suffix in suffixes)]


# Beginning of computer generated data:
commands = {
    'atom':
        ['-f', '--xc-functional', '-a', '--add', '--spin-polarized', '-d',
         '--dirac', '-p', '--plot', '-e', '--exponents', '-l',
         '--logarithmic-derivatives', '-r', '--refine', '-s',
         '--scalar-relativistic', '--no-ee-interaction'],
    'completion':
        [],
    'dataset':
        ['-f', '--xc-functional', '-C', '--configuration', '-P',
         '--projectors', '-r', '--radius', '-0',
         '--zero-potential', '-c',
         '--pseudo-core-density-radius', '-z', '--pseudize',
         '-p', '--plot', '-l', '--logarithmic-derivatives', '-w',
         '--write', '-s', '--scalar-relativistic', '-n',
         '--no-check', '-t', '--tag', '-a', '--alpha', '-g',
         '--gamma', '-b', '--create-basis-set', '--nlcc',
         '--core-hole', '-e', '--electrons', '-o', '--output'],
    'diag':
        ['-b', '--bands', '-s', '--scalapack', '-0', '--dry-run'],
    'dos':
        ['-p', '--plot', '-i', '--integrated', '-w', '--width', '-a',
         '--atom', '-t', '--total', '-r', '--range', '-n',
         '--points'],
    'gpw':
        ['-w', '--remove-wave-functions'],
    'info':
        [],
    'install-data':
        ['--version', '--tarball', '--list-all', '--gpaw', '--sg15',
         '--basis', '--test', '--register', '--no-register'],
    'python':
        [],
    'rpa':
        ['-e', '--cut-off', '-b', '--blocks'],
    'run':
        ['-p', '--parameters', '-t', '--tag', '--properties', '-f',
         '--maximum-force', '--constrain-tags', '-s',
         '--maximum-stress', '-E', '--equation-of-state',
         '--eos-type', '--modify', '--after', '-i',
         '--interactive', '-c', '--collection', '-d',
         '--database', '-S', '--skip', '-w', '--write', '-W',
         '--write-all'],
    'sbatch':
        ['-0', '--dry-run'],
    'symmetry':
        [],
    'test':
        ['-x', '--exclude', '-f', '--run-failed-tests-only', '--from',
         '--after', '--range', '-j', '--jobs', '--reverse', '-k',
         '--keep-temp-dir', '-d', '--directory', '-s',
         '--show-output', '--list']}
# End of computer generated data


def complete(word, previous, line, point):
    for w in line[:point - len(word)].strip().split()[1:]:
        if w[0].isalpha():
            if w in commands:
                command = w
                break
    else:
        if word[:1] == '-':
            return ['-h', '--help', '--version', '-P', '--parallel']
        return list(commands.keys()) + ['-h', '--help', '--version',
                                        '-P', '--parallel']

    if word[:1] == '-':
        return commands[command]

    words = []

    if command == 'help':
        words = commands

    elif command == 'test':
        from glob import glob
        path = __file__.rsplit('/', 1)[0] + '/../test/**/*.py'
        return (word.rsplit('/test/')[-1]
                for word in glob(path, recursive=True))

    return words


word, previous = sys.argv[2:]
line = os.environ['COMP_LINE']
point = int(os.environ['COMP_POINT'])
words = complete(word, previous, line, point)
for w in words:
    if w.startswith(word):
        print(w)



"""Test of ScaLAPACK diagonalize and inverse cholesky.

The test generates a random symmetric matrix H0 and
positive definite matrix S0 on a 1-by-1 BLACS grid. They
are redistributed to a mprocs-by-nprocs BLACS grid,
diagonalized in parallel, and eigenvalues are compared
against LAPACK. Eigenvectors are not compared.
"""
from __future__ import print_function

import numpy as np
from numpy.linalg import inv
from gpaw.mpi import world, rank
from gpaw.blacs import BlacsGrid, Redistributor
from gpaw.utilities.tools import tri2full
from gpaw.utilities import compiled_with_sl
from gpaw.utilities.lapack import diagonalize, general_diagonalize, \
    inverse_cholesky
from gpaw.utilities.blas import rk
from gpaw.utilities.scalapack import scalapack_general_diagonalize_dc, \
    scalapack_diagonalize_dc, \
    scalapack_inverse_cholesky, scalapack_inverse

tol = 1.0e-8

def main(N=72, seed=42, mprocs=2, nprocs=2, dtype=float):
    gen = np.random.RandomState(seed)
    grid = BlacsGrid(world, mprocs, nprocs)

    if (dtype==complex):
        epsilon = 1.0j
    else:
        epsilon = 0.0

    # Create descriptors for matrices on master:
    glob = grid.new_descriptor(N, N, N, N)

    # print globA.asarray()
    # Populate matrices local to master:
    H0 = glob.zeros(dtype=dtype) + gen.rand(*glob.shape)
    S0 = glob.zeros(dtype=dtype) + gen.rand(*glob.shape)
    C0 = glob.empty(dtype=dtype)
    if rank == 0:
        # Complex case must have real numbers on the diagonal.
        # We make a simple complex Hermitian matrix below.
        H0 = H0 + epsilon * (0.1*np.tri(N, N, k= -N // nprocs) + 0.3*np.tri(N, N, k=-1))
        S0 = S0 + epsilon * (0.2*np.tri(N, N, k= -N // nprocs) + 0.4*np.tri(N, N, k=-1))
        # Make matrices symmetric
        rk(1.0, H0.copy(), 0.0, H0)
        rk(1.0, S0.copy(), 0.0, S0)
        # Overlap matrix must be semi-positive definite
        S0 = S0 + 50.0*np.eye(N, N, 0)
        # Hamiltonian is usually diagonally dominant
        H0 = H0 + 75.0*np.eye(N, N, 0)
        C0 = S0.copy()
        S0_inv = S0.copy()

    # Local result matrices
    W0 = np.empty((N),dtype=float)
    W0_g = np.empty((N),dtype=float)

    # Calculate eigenvalues / other serial results
    if rank == 0:
        diagonalize(H0.copy(), W0)
        general_diagonalize(H0.copy(), W0_g, S0.copy())
        inverse_cholesky(C0) # result returned in lower triangle
        tri2full(S0_inv, 'L')
        S0_inv = inv(S0_inv)
        # tri2full(C0) # symmetrize

    assert glob.check(H0) and glob.check(S0) and glob.check(C0)

    # Create distributed destriptors with various block sizes:
    dist = grid.new_descriptor(N, N, 8, 8)

    # Distributed matrices:
    # We can use empty here, but end up with garbage on
    # on the other half of the triangle when we redistribute.
    # This is fine because ScaLAPACK does not care.

    H = dist.empty(dtype=dtype)
    S = dist.empty(dtype=dtype)
    Sinv = dist.empty(dtype=dtype)
    Z = dist.empty(dtype=dtype)
    C = dist.empty(dtype=dtype)
    Sinv = dist.empty(dtype=dtype)

    # Eigenvalues are non-BLACS matrices
    # W = np.empty((N), dtype=float)
    W_dc = np.empty((N), dtype=float)
    # W_mr3 = np.empty((N), dtype=float)
    # W_g = np.empty((N), dtype=float)
    W_g_dc = np.empty((N), dtype=float)
    # W_g_mr3 = np.empty((N), dtype=float)

    Glob2dist = Redistributor(world, glob, dist)
    Glob2dist.redistribute(H0, H, uplo='L')
    Glob2dist.redistribute(S0, S, uplo='L')
    Glob2dist.redistribute(S0, C, uplo='L') # C0 was previously overwritten
    Glob2dist.redistribute(S0, Sinv, uplo='L')

    # we don't test the expert drivers anymore since there
    # might be a buffer overflow error
    ## scalapack_diagonalize_ex(dist, H.copy(), Z, W, 'L')
    scalapack_diagonalize_dc(dist, H.copy(), Z, W_dc, 'L')
    ## scalapack_diagonalize_mr3(dist, H.copy(), Z, W_mr3, 'L')
    ## scalapack_general_diagonalize_ex(dist, H.copy(), S.copy(), Z, W_g, 'L')
    scalapack_general_diagonalize_dc(dist, H.copy(), S.copy(), Z, W_g_dc, 'L')
    ## scalapack_general_diagonalize_mr3(dist, H.copy(), S.copy(), Z, W_g_mr3, 'L')

    scalapack_inverse_cholesky(dist, C, 'L')

    if dtype == complex: # Only supported for complex for now
        scalapack_inverse(dist, Sinv, 'L')
    # Undo redistribute
    C_test = glob.empty(dtype=dtype)
    Sinv_test = glob.empty(dtype=dtype)
    Dist2glob = Redistributor(world, dist, glob)
    Dist2glob.redistribute(C, C_test)
    Dist2glob.redistribute(Sinv, Sinv_test)

    if rank == 0:
        ## diag_ex_err = abs(W - W0).max()
        diag_dc_err = abs(W_dc - W0).max()
        ## diag_mr3_err = abs(W_mr3 - W0).max()
        ## general_diag_ex_err = abs(W_g - W0_g).max()
        general_diag_dc_err = abs(W_g_dc - W0_g).max()
        ## general_diag_mr3_err = abs(W_g_mr3 - W0_g).max()
        inverse_chol_err = abs(C_test-C0).max()

        tri2full(Sinv_test,'L')
        inverse_err = abs(Sinv_test - S0_inv).max()
        ## print 'diagonalize ex err', diag_ex_err
        print('diagonalize dc err', diag_dc_err)
        ## print 'diagonalize mr3 err', diag_mr3_err
        ## print 'general diagonalize ex err', general_diag_ex_err
        print('general diagonalize dc err', general_diag_dc_err)
        ## print 'general diagonalize mr3 err', general_diag_mr3_err
        print('inverse chol err', inverse_chol_err)
        if dtype == complex:
            print('inverse err', inverse_err)
    else:
        ## diag_ex_err = 0.0
        diag_dc_err = 0.0
        ## diag_mr3_err = 0.0
        ## general_diag_ex_err = 0.0
        general_diag_dc_err = 0.0
        ## general_diag_mr3_err = 0.0
        inverse_chol_err = 0.0
        inverse_err = 0.0

    # We don't like exceptions on only one cpu
    ## diag_ex_err = world.sum(diag_ex_err)
    diag_dc_err = world.sum(diag_dc_err)
    ## diag_mr3_err = world.sum(diag_mr3_err)
    ## general_diag_ex_err = world.sum(general_diag_ex_err)
    general_diag_dc_err = world.sum(general_diag_dc_err)
    ## general_diag_mr3_err = world.sum(general_diag_mr3_err)
    inverse_chol_err = world.sum(inverse_chol_err)
    inverse_err = world.sum(inverse_err)
    ## assert diag_ex_err < tol
    assert diag_dc_err < tol
    ## assert diag_mr3_err < tol
    ## assert general_diag_ex_err < tol
    assert general_diag_dc_err < tol
    ## assert general_diag_mr3_err < tol
    assert inverse_chol_err < tol
    if dtype == complex:
        assert inverse_err < tol

if __name__ in ['__main__', '__builtin__']:
    if not compiled_with_sl():
        print('Not built with ScaLAPACK. Test does not apply.')
    else:
        main(dtype=complex)
        main(dtype=float)






import numpy as np
from gpaw.blacs import BlacsGrid
from gpaw.mpi import world, rank, size
from gpaw.utilities.lapack import diagonalize
from gpaw.blacs import Redistributor


def scal_diagonalize(A, nodes='master'):
    # Diagonalize matrix A (size N*N) with scalapack
    # Usage: eps, B = scal_diagonalize(A)
    # eps and B and the eigenvalues and eigenvectors
    # nodes = 'master': eigenvectors only available on master node
    # nodes = 'all': eigenvectors broadcast to all nodes

    # make sure A is N*N, and hermitian
    N = A.shape[0]
    assert A.shape[0] == A.shape[1]
    for i in range(N):
        for j in range(i, N):
            assert A[i,j] == A[j,i].conj()

    # create blacs descriptor
    mb = 64
    g = BlacsGrid(world, 2, size//2)
    nndesc1 = g.new_descriptor(N, N, N,  N)
    nndesc2 = g.new_descriptor(N, N, mb, mb)

    # distribute A to blacs grid A_
    if rank != 0:
        A = nndesc1.zeros(dtype=A.dtype)
    A_ = nndesc2.empty(dtype=A.dtype)
    redistributor = Redistributor(world, nndesc1, nndesc2)
    redistributor.redistribute(A, A_)

    # diagonalize
    B_ = nndesc2.zeros(dtype=A.dtype)
    eps = np.zeros(N,dtype=A.dtype)
    nndesc2.diagonalize_dc(A_, B_, eps, 'L')

    # distribute the eigenvectors to master
    B = np.zeros_like(A)
    redistributor = Redistributor(world, nndesc2, nndesc1)
    redistributor.redistribute(B_, B)

    if nodes == 'master':
        return eps, B
    elif nodes == 'all':
        if rank != 0:
            B = np.zeros((N, N))
        world.broadcast(B, 0)
        return eps, B


# generate a matrix
N = 512
A = np.arange(N**2,dtype=float).reshape(N,N)
for i in range(N):
    for j in range(i,N):
        A[i,j] = A[j,i]

# diagonalize
eps, B = scal_diagonalize(A)


check = 1
if check and rank == 0:
    # check whether it gives the same result with lapack
    eps1 = np.zeros(N)
    diagonalize(A, eps1)
    assert np.abs(eps-eps1).sum() < 1e-6

    for i in range(N//size):
        # the eigenvectors are row of the matrix, it can be differ by a minus sign.
        if np.abs(A[i,:] - B[i,:]).sum() > 1e-6:
            if np.abs(A[i,:] + B[i,:]).sum() > 1e-6:
                raise ValueError('Check !')




from __future__ import print_function
from ase import Atom

from gpaw import GPAW, FermiDirac, PoissonSolver
from gpaw.cluster import Cluster
from gpaw.test import equal

h =.35
box = 3.
energy_tolerance = 0.0004

s = Cluster([Atom('Zn')])
s.minimal_box(box, h=h)

E = {}
E_U = {}
for spin in [0, 1]:
    c = GPAW(h=h, spinpol=spin,
             xc='oldLDA',
             mode='lcao', basis='sz(dzp)',
             poissonsolver=PoissonSolver(eps=1e-7),
             parallel=dict(kpt=1),
             charge=1, occupations=FermiDirac(width=0.1, fixmagmom=spin)
             )
    s.set_calculator(c)
    E[spin] = s.get_potential_energy()
    c.set(setups=':d,3.0,1')
    E_U[spin] = s.get_potential_energy()

print("E=", E)
equal(E[0], E[1], energy_tolerance)
print("E_U=", E_U)
equal(E_U[0], E_U[1], energy_tolerance)



# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

import numpy as np
from gpaw.grid_descriptor import GridDescriptor
from gpaw.transformers import Transformer


n = 20
gd = GridDescriptor((n,n,n))
np.random.seed(8)
a = gd.empty()
a[:] = np.random.random(a.shape)

gd2 = gd.refine()
b = gd2.zeros()
for k in [2, 4, 6, 8]:
    inter = Transformer(gd, gd2, k // 2).apply
    inter(a, b)
    assert abs(gd.integrate(a) - gd2.integrate(b)) < 1e-14

gd2 = gd.coarsen()
b = gd2.zeros()
for k in [2, 4, 6, 8]:
    restr = Transformer(gd, gd2, k // 2).apply
    restr(a, b)
    assert abs(gd.integrate(a) - gd2.integrate(b)) < 1e-14

# complex versions
a = gd.empty(dtype=complex)
a.real = np.random.random(a.shape)
a.imag = np.random.random(a.shape)

phase = np.ones((3, 2), complex)

gd2 = gd.refine()
b = gd2.zeros(dtype=complex)
for k in [2, 4, 6, 8]:
    inter = Transformer(gd, gd2, k // 2, complex).apply
    inter(a, b, phase)
    assert abs(gd.integrate(a) - gd2.integrate(b)) < 1e-14

gd2 = gd.coarsen()
b = gd2.zeros(dtype=complex)
for k in [2, 4, 6, 8]:
    restr = Transformer(gd, gd2, k // 2, complex).apply
    restr(a, b, phase)
    assert abs(gd.integrate(a) - gd2.integrate(b)) < 1e-14





from ase import Atom 

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.analyse.eed import ExteriorElectronDensity

fwfname='H2_kpt441_wf.gpw'
txt = None
#txt = '-'

# write first if needed
try:
    c = GPAW(fwfname, txt=txt)
    s = c.get_atoms()
except:
    s = Cluster([Atom('H'), Atom('H', [0,0,1])], pbc=[1,1,0])
    s.minimal_box(3.)
    c = GPAW(xc='PBE', h=.3, kpts=(4,4,1),
             convergence={'density':1e-3, 'eigenstates':1e-3})
    c.calculate(s)
    c.write(fwfname, 'all')

eed = ExteriorElectronDensity(c.wfs.gd, s)
eed.write_mies_weights(c.wfs)



from ase.spacegroup import crystal
from gpaw import GPAW
from gpaw import PW
from gpaw.test import equal

name = 'sishovite'
# no 136 - tetragonal

a = 4.233944
c = 2.693264
p0 = (0, 0, 0)
p1 = (0.306866, 0.306866, 0.0)

atoms = crystal(['Si', 'O'], basis=[p0, p1],
                spacegroup=136, cellpar=[a, a, c, 90, 90, 90])


## with fractional translation
calc = GPAW(mode=PW(),
            xc='LDA',
            kpts=(3, 3, 3),
            nbands=28,
            symmetry={'symmorphic': False},
            gpts=(18, 18, 12),
            eigensolver='rmm-diis')

atoms.set_calculator(calc)
energy_fractrans = atoms.get_potential_energy()

assert(len(calc.wfs.kd.ibzk_kc) == 6)
assert(len(calc.wfs.kd.symmetry.op_scc) == 16)

## without fractional translations
calc = GPAW(mode=PW(),
            xc='LDA',
            kpts=(3, 3, 3),
            nbands=28,
            gpts=(18, 18, 12),
            eigensolver='rmm-diis')

atoms.set_calculator(calc)
energy_no_fractrans = atoms.get_potential_energy()

assert(len(calc.wfs.kd.ibzk_kc) == 8)
assert(len(calc.wfs.kd.symmetry.op_scc) == 8)

equal(energy_fractrans, energy_no_fractrans, 1e-7)



"""
tests solvation parameters as in

V. M. Sanchez, M. Sued, and D. A. Scherlis,
The Journal of Chemical Physics, vol. 131, no. 17, p. 174108, 2009
"""

from gpaw import GPAW
from gpaw.cluster import Cluster
from gpaw.test import equal
from ase import Atoms
from ase.units import mol, kcal, Pascal, m, Bohr
from gpaw.solvation import (
    SolvationGPAW,
    FG02SmoothStepCavity,
    LinearDielectric,
    GradientSurface,
    SurfaceInteraction,
    SSS09Density)

SKIP_VAC_CALC = True

h = 0.24
vac = 4.0

epsinf = 78.36
rho0 = 1.0 / Bohr ** 3
beta = 2.4
st = 72. * 1e-3 * Pascal * m
atomic_radii = lambda atoms: [2.059]
convergence = {
    'energy': 0.05 / 8.,
    'density': 10.,
    'eigenstates': 10.,
}

atoms = Cluster(Atoms('Cl'))
atoms.minimal_box(vac, h)

if not SKIP_VAC_CALC:
    atoms.calc = GPAW(xc='PBE', h=h, charge=-1, convergence=convergence)
    Evac = atoms.get_potential_energy()
    print(Evac)
else:
    # h=0.24, vac=4.0, setups: 0.9.11271, convergence: only energy 0.05 / 8
    Evac = -3.83245253419

atoms.calc = SolvationGPAW(
    xc='PBE', h=h, charge=-1, convergence=convergence,
    cavity=FG02SmoothStepCavity(
        rho0=rho0, beta=beta,
        density=SSS09Density(atomic_radii=atomic_radii),
        surface_calculator=GradientSurface()
    ),
    dielectric=LinearDielectric(epsinf=epsinf),
    interactions=[SurfaceInteraction(surface_tension=st)]
)
Ewater = atoms.get_potential_energy()
assert atoms.calc.get_number_of_iterations() < 40
atoms.get_forces()
DGSol = (Ewater - Evac) / (kcal / mol)
print('Delta Gsol: %s kcal / mol' % DGSol)

equal(DGSol, -75., 10.)



import numpy as np

from gpaw.sphere import lmfact
from gpaw.sphere.legendre import ilegendre, legendre, dlegendre

# Define the Heaviside function
heaviside = lambda x: (1.0+np.sign(x))/2.0

# Define spherical harmoncics and normalization coefficient
C = lambda l,m: ((2.*l+1.)/(4*np.pi*lmfact(l,m)))**0.5

def Y(l,m,theta,phi):
    if m == 0:
        return C(l,m)*legendre(l,abs(m),np.cos(theta))
    elif m > 0:
        return C(l,m)*legendre(l,abs(m),np.cos(theta))*np.cos(m*phi)*2**0.5
    else:
        return C(l,m)*legendre(l,abs(m),np.cos(theta))*np.sin(abs(m)*phi)*2**0.5

# Define theta-derivative of spherical harmoncics
def dYdtheta(l,m,theta,phi):
    if m == 0:
        return C(l,m)*dlegendre(l,abs(m),np.cos(theta))
    elif m > 0:
        return C(l,m)*dlegendre(l,abs(m),np.cos(theta))*np.cos(m*phi)*2**0.5
    else:
        return C(l,m)*dlegendre(l,abs(m),np.cos(theta))*np.sin(abs(m)*phi)*2**0.5

# Define phi-derivative of spherical harmoncics
def dYdphi(l,m,theta,phi):
    if m == 0:
        return np.zeros_like(theta)
    elif m > 0:
        return -m*C(l,m)*legendre(l,abs(m),np.cos(theta))*np.sin(m*phi)*2**0.5
    else:
        return -m*C(l,m)*legendre(l,abs(m),np.cos(theta))*np.cos(m*phi)*2**0.5

# -------------------------------------------------------------------

def intYY(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /
           A    = |  |    y (u,v) y (u,v) sin(u) dv du
            LL'   /  /     lm      l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if l1 = l2 and m1 = m2.
    """
    if (l1,m1) == (l2,m2):
        return C(l1,m1)**2*2*np.pi*ilegendre(l1,m1) # == 1 always, XXX right?
    else:
        return 0.0

# -------------------------------------------------------------------

from gpaw.sphere.csh import intYY_ex as _intYY_ex, intYY_ey as _intYY_ey, intYY_ez as _intYY_ez #TODO make independent

def mix(l1, m1, l2, m2, func):
    # m1 == 0
    if m1 == 0 and m2 == 0:
        return func(l1,abs(m1),l2,abs(m2))
    elif m1 == 0 and m2 > 0:
        return 1/2**0.5*((-1)**m2*func(l1,abs(m1),l2,abs(m2)) \
                        +func(l1,abs(m1),l2,-abs(m2)))
    elif m1 == 0 and m2 < 0:
        return 1/(2**0.5*1j)*((-1)**m2*func(l1,abs(m1),l2,abs(m2)) \
                             -func(l1,abs(m1),l2,-abs(m2)))
    # m1 > 0
    elif m1 > 0 and m2 == 0:
        return 1/2**0.5*((-1)**m1*func(l1,abs(m1),l2,abs(m2)) \
                        +func(l1,-abs(m1),l2,abs(m2)))
    elif m1 > 0 and m2 > 0:
        return 0.5*((-1)**(m1+m2)*func(l1,abs(m1),l2,abs(m2)) \
                   +(-1)**m1*func(l1,abs(m1),l2,-abs(m2)) \
                   +(-1)**m2*func(l1,-abs(m1),l2,abs(m2)) \
                   +func(l1,-abs(m1),l2,-abs(m2)))
    elif m1 > 0 and m2 < 0:
        return 1/2j*((-1)**(m1+m2)*func(l1,abs(m1),l2,abs(m2)) \
                    -(-1)**m1*func(l1,abs(m1),l2,-abs(m2)) \
                    +(-1)**m2*func(l1,-abs(m1),l2,abs(m2)) \
                    -func(l1,-abs(m1),l2,-abs(m2)))
    # m1 < 0
    elif m1 < 0 and m2 == 0:
        return -1/(2**0.5*1j)*((-1)**m1*func(l1,abs(m1),l2,abs(m2)) \
                             -func(l1,-abs(m1),l2,abs(m2)))
    elif m1 < 0 and m2 > 0:
        return -1/2j*((-1)**(m1+m2)*func(l1,abs(m1),l2,abs(m2)) \
                     +(-1)**m1*func(l1,abs(m1),l2,-abs(m2)) \
                     -(-1)**m2*func(l1,-abs(m1),l2,abs(m2)) \
                     -func(l1,-abs(m1),l2,-abs(m2)))
    elif m1 < 0 and m2 < 0:
        return 0.5*((-1)**(m1+m2)*func(l1,abs(m1),l2,abs(m2)) \
                   -(-1)**m1*func(l1,abs(m1),l2,-abs(m2)) \
                   -(-1)**m2*func(l1,-abs(m1),l2,abs(m2)) \
                   +func(l1,-abs(m1),l2,-abs(m2)))
    else:
        raise ValueError('Invalid arguments (l1=%s, m1=%s, l2=%s, m2=%s)' \
                         % (l1,m1,l2,m2))

def intYY_ex(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /
           A    = |  |    y (u,v) sin(u)*cos(v) y (u,v) sin(u) dv du
            LL'   /  /     lm                    l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and `|m1-m2|` = 1.
    """
    #if abs(l1-l2) != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = C(l1,m1)*C(l2,m2)*np.pi
    #if abs(m1) == abs(m2)+1:
    #    return scale*np.sign(l1-l2)/(2.*l2+1.)*ilegendre(l1,m1)
    #else:
    #    assert abs(m1) == abs(m2)-1
    #    return scale*np.sign(l2-l1)/(2.*l1+1.)*ilegendre(l2,m2)
    return mix(l1,m1,l2,m2,_intYY_ex)

def intYY_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /
           A    = |  |    y (u,v) sin(u)*sin(v) y (u,v) sin(u) dv du
            LL'   /  /     lm                    l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and `|m1-m2|` = 1.
    """
    #if abs(l1-l2) != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j
    #if abs(m1) == abs(m2)+1:
    #    return scale*np.sign(l1-l2)/(2.*l2+1.)*ilegendre(l1,m1)
    #else:
    #    assert abs(m1) == abs(m2)-1
    #    return scale*np.sign(l2-l1)/(2.*l1+1.)*ilegendre(l2,m2)
    return mix(l1,m1,l2,m2,_intYY_ey)

def intYY_ez(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /
           A    = |  |    y (u,v) cos(u) y (u,v) sin(u) dv du
            LL'   /  /     lm             l'm'
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and m1 = m2.
    """
    return mix(l1,m1,l2,m2,_intYY_ez)

# -------------------------------------------------------------------

from gpaw.sphere.csh import intYdYdtheta_ex as _intYdYdtheta_ex, intYdYdtheta_ey as _intYdYdtheta_ey, intYdYdtheta_ez as _intYdYdtheta_ez #TODO make independent

def intYdYdtheta_ex(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /                           d y(u,v)
           A    = |  |    y (u,v) cos(u)*cos(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    du  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    #if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = -C(l1,m1)*C(l2,m2)*np.pi
    #if abs(m1) == abs(m2)+1:
    #    if l1+1 == l2:
    #        return scale*2/(2.0*l2+1)*(l2+1)/(2.0*l2-1.0)*fact(l2+abs(m2))/fact(l2-abs(m2)-1)*(l2-abs(m2)-1)
    #    elif l1-1 == l2:
    #        return scale*2/(2.0*l2+1)*((l2+1)*fact(l2+abs(m2))/fact(l2-abs(m2))*(l2-abs(m2))-l2/(2.0*l2+3.0)*fact(l2+abs(m2)+1)/fact(l2-abs(m2))*(l2-abs(m2)+1))
    #    elif l1-l2 > 2: # and (l1-l2)%2 == 1 which is always true
    #        return -scale*2*abs(m2)*fact(l2+abs(m2))/fact(l2-abs(m2))
    #    else:
    #        return 0.0
    #else:
    #    assert abs(m1) == abs(m2)-1
    #    if l1 == l2+1:
    #        return -scale*2/(2.0*l1+1.0)*(l1-1)/(2.0*l1-1.0)*fact(l1+abs(m1))/fact(l1-abs(m1)-1)*(l1-abs(m1)-1)
    #    elif l1 == l2-1:
    #        return -scale*2/(2.0*l1+1.0)*((l1+1)/(2.0*l1+3.0)*fact(l1+abs(m1)+1)/fact(l1-abs(m1))*(l1-abs(m1)+1)-(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))*(l1-abs(m1)))
    #    elif l2-l1 > 2: # and (l2-l1)%2 == 1 which is always true
    #        return scale*2*(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))
    #    else:
    #        return 0.0
    return mix(l1,m1,l2,m2,_intYdYdtheta_ex)

def intYdYdtheta_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /                           d y(u,v)
           A    = |  |    y (u,v) cos(u)*sin(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    du  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    #if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = -C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j
    #if abs(m1) == abs(m2)+1:
    #    if l1+1 == l2:
    #        return scale*2/(2.0*l2+1)*(l2+1)/(2.0*l2-1.0)*fact(l2+abs(m2))/fact(l2-abs(m2)-1)*(l2-abs(m2)-1)
    #    elif l1-1 == l2:
    #        return scale*2/(2.0*l2+1)*((l2+1)*fact(l2+abs(m2))/fact(l2-abs(m2))*(l2-abs(m2))-l2/(2.0*l2+3.0)*fact(l2+abs(m2)+1)/fact(l2-abs(m2))*(l2-abs(m2)+1))
    #    elif l1-l2 > 2: # and (l1-l2)%2 == 1 which is always true
    #        return -scale*2*abs(m2)*fact(l2+abs(m2))/fact(l2-abs(m2))
    #    else:
    #        return 0.0
    #else:
    #    assert abs(m1) == abs(m2)-1
    #    if l1 == l2+1:
    #        return -scale*2/(2.0*l1+1.0)*(l1-1)/(2.0*l1-1.0)*fact(l1+abs(m1))/fact(l1-abs(m1)-1)*(l1-abs(m1)-1)
    #    elif l1 == l2-1:
    #        return -scale*2/(2.0*l1+1.0)*((l1+1)/(2.0*l1+3.0)*fact(l1+abs(m1)+1)/fact(l1-abs(m1))*(l1-abs(m1)+1)-(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))*(l1-abs(m1)))
    #    elif l2-l1 > 2: # and (l2-l1)%2 == 1 which is always true
    #        return scale*2*(abs(m1)+1)*fact(l1+abs(m1))/fact(l1-abs(m1))
    #    else:
    #        return 0.0
    return mix(l1,m1,l2,m2,_intYdYdtheta_ey)

def intYdYdtheta_ez(l1, m1, l2, m2):
    """Calculates::

                     pi 2pi
                     /  /                    d y(u,v)
           A    =  - |  |    y (u,v) sin(u) --- l'm'  sin(u) dv du
            LL'      /  /     lm             du  
                     0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` = 1 and m1 = m2.
    """
    #if abs(l1-l2) != 1 or m1 != m2:
    #    return 0.0
    #scale = -C(l1,m1)*C(l2,m2)*2*np.pi
    #return scale*np.sign(l1-l2)*(l2+1-heaviside(l1-l2))*(l2-np.sign(l1-l2)*abs(m1)+heaviside(l1-l2))/(2*l2+1)*ilegendre(l1,m1)
    return mix(l1,m1,l2,m2,_intYdYdtheta_ez)

# -------------------------------------------------------------------

from gpaw.sphere.csh import intYdYdphi_ex as _intYdYdphi_ex, intYdYdphi_ey as _intYdYdphi_ey #TODO make independent

def intYdYdphi_ex(l1, m1, l2, m2):
    """Calculates::

                     pi 2pi
                     /  /              -1           d y(u,v)
           A    =  - |  |    y (u,v) sin(u)*sin(v) --- l'm'  sin(u) dv du
            LL'      /  /     lm                    dv  
                     0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    #if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = -C(l1,m1)*C(l2,m2)*np.sign(m1-m2)*np.pi/1j*1j*m2
    #if abs(m1) == abs(m2)+1 and l1 > l2: # and (l1-l2)%2 == 1 which is always true
    #    return scale*2*lmfact(l2,m2)
    #elif abs(m1) == abs(m2)-1 and l1 < l2: # and (l2-l1)%2 == 1 which is always true
    #    return scale*2*lmfact(l1,m1)
    #else:
    #    return 0.0
    return mix(l1,m1,l2,m2,_intYdYdphi_ex)

def intYdYdphi_ey(l1, m1, l2, m2):
    """Calculates::

                  pi 2pi
                  /  /              -1           d y(u,v)
           A    = |  |    y (u,v) sin(u)*cos(v) --- l'm'  sin(u) dv du
            LL'   /  /     lm                    dv  
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` = 1 (stricter rule applies).
    """
    #if abs(l1-l2) % 2 != 1 or abs(m1-m2) != 1:
    #    return 0.0
    #scale = C(l1,m1)*C(l2,m2)*np.pi*1j*m2
    #if abs(m1) == abs(m2)+1 and l1 > l2: # and (l1-l2)%2 == 1 which is always true
    #    return scale*2*lmfact(l2,m2)
    #elif abs(m1) == abs(m2)-1 and l1 < l2: # and (l2-l1)%2 == 1 which is always true
    #    return scale*2*lmfact(l1,m1)
    #else:
    #    return 0.0
    return mix(l1,m1,l2,m2,_intYdYdphi_ey)

def intYdYdphi_ez(l1, m1, l2, m2): #XXX this is silly
    return 0.0

# -------------------------------------------------------------------

def intYgradY(l1, m1, l2, m2, r_g, dr_g, A_g, B_g, dBdr_g, v=None):
    """Calculates::

                  pi 2pi
                  /  /    *           / __               \  2
          A     = |  |   A(r) y (u,v) | \/  B(r) y (u,v) | r sin(u) dv du dr
           vLL'   /  /         lm     \   v       l'm'   /
                  0  0

    where u = theta and v = phi in the usual notation. Note that the result
    is only non-zero if `|l1-l2|` is odd and `|m1-m2|` <= 1 (stricter rule applies).
    """
    if v is None:
        D = [intYY_ex(l1,m1,l2,m2), \
             intYY_ey(l1,m1,l2,m2), \
             intYY_ez(l1,m1,l2,m2)]
        G = [intYdYdtheta_ex(l1,m1,l2,m2) + intYdYdphi_ex(l1,m1,l2,m2), \
             intYdYdtheta_ey(l1,m1,l2,m2) + intYdYdphi_ey(l1,m1,l2,m2), \
             intYdYdtheta_ez(l1,m1,l2,m2) + intYdYdphi_ez(l1,m1,l2,m2)]
        D, G = np.array(D), np.array(G)
    elif v == 0:
        D = intYY_ex(l1,m1,l2,m2)
        G = intYdYdtheta_ex(l1,m1,l2,m2) + intYdYdphi_ex(l1,m1,l2,m2)
    elif v == 1:
        D = intYY_ey(l1,m1,l2,m2)
        G = intYdYdtheta_ey(l1,m1,l2,m2) + intYdYdphi_ey(l1,m1,l2,m2)
    elif v == 2:
        D = intYY_ez(l1,m1,l2,m2)
        G = intYdYdtheta_ez(l1,m1,l2,m2) + intYdYdphi_ez(l1,m1,l2,m2)
    else:
        raise ValueError
    return D * np.vdot(A_g, dBdr_g * r_g**2 * dr_g) \
        + G * np.vdot(A_g, B_g * r_g * dr_g)




from gpaw.solvation.sjm import SJM, SJMPower12Potential

from ase.build import fcc111
from gpaw import FermiDirac

# Import solvation modules
from ase.data.vdw import vdw_radii
from gpaw.solvation import (
    EffectivePotentialCavity,
    LinearDielectric,
    GradientSurface,
    SurfaceInteraction
)

# Solvent parameters
u0 = 0.180  # eV
epsinf = 78.36  # Dielectric constant of water at 298 K
gamma = 0.00114843767916  # 18.4*1e-3 * Pascal* m
T = 298.15   # K
atomic_radii = lambda atoms: [vdw_radii[n] for n in atoms.numbers]

# Structure is created
atoms = fcc111('Au', size=(1, 1, 3))
atoms.center(axis=2, vacuum=8)
atoms.translate([0, 0, -2])

# SJM parameters
potential = 3.0
ne = 1.20
dpot = 0.05

# The calculator
calc = SJM(doublelayer={'upper_limit': 19.5},
           potential=potential,
           dpot=dpot,
           ne=ne,
           gpts=(8, 8, 64),
           poissonsolver={'dipolelayer': 'xy'},
           kpts=(1, 1, 1),
           xc='PBE',
           occupations=FermiDirac(0.1),
           cavity=EffectivePotentialCavity(
               effective_potential=SJMPower12Potential(atomic_radii, u0),
               temperature=T,
               surface_calculator=GradientSurface()),
           dielectric=LinearDielectric(epsinf=epsinf),
           interactions=[SurfaceInteraction(surface_tension=gamma)])

# Run the calculation
atoms.set_calculator(calc)
atoms.get_potential_energy()
assert abs(calc.get_electrode_potential() - potential) < dpot
elpot = calc.get_electrostatic_potential().mean(0).mean(0)
assert abs(elpot[2] - elpot[10]) < 1e-3
